/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.api;

import java.util.EventObject;

/**
 * This event is sent when a persistent class is mapped.
 *
 * @author S.Chassande-Barrioz
 */
public class ClassMappedEvent extends EventObject {

	/**
	 * The name of the persistent class which has been mapped.
	 */
	private String jormClassName;

	/**
	 * @param mapper is the mapper containing the new persistent class. The
	 * mapper is the source of the event.
	 * @param jormClassName is the name of the persistent class which has
	 * been mapped.
	 */
	public ClassMappedEvent(PMapper mapper, String jormClassName) {
		super(mapper);
		this.jormClassName = jormClassName;
	}

	/**
	 * @return the mapper containing the new persistent class. The
	 * mapper is the source of the event.
	 */
	public PMapper getPMapper() {
		return (PMapper) source;
	}

	/**
	 * @return The name of the persistent class which has been mapped.
	 */
	public String getJormClassName() {
		return jormClassName;
	}
}
