/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.api;

import java.util.EventObject;

/**
 * The event used to signals the requirement of the mapping of a particular
 * JORM class.
 * @author P. Dechamboux
 */
public class PMappingRequiredEvent extends EventObject {
	private String jormClassName;

	/**
	 * Constructs a PMappingRequiredEvent event.
	 * @param source        The event source.
	 * @param jormclassname The name of the JORM class that is required to be
	 *                      mapped.
	 */
	public PMappingRequiredEvent(PClassMapping source, String jormclassname) {
		super(source);
		jormClassName = jormclassname;
	}

	/**
	 * Retrives the name of the JORM class that is required to be mapped.
	 * @return  This class name.
	 */
	public String getJormClassName() {
		return jormClassName;
	}
}
