/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.api;

import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;

/**
 * Defines an extension of a TupleCollection to use for the prefetching
 * mechanism. This extension permits to known the index of the PName of the
 * persistent class which is pefetched. It permits also to reach the prefetched
 * buffer encaspsulates behind this tuple collection.
 *
 * @author P.Dechamboux
 * @author S.Chassande-Barrioz
 *
 */
public interface PPrefetchTupleCollection extends TupleCollection {
	/**
	 * Gets the index of the PName of the persistent class which is pefetched
	 * @return an integer value greater or equal to 1.
	 */
	int getPNameIndex();

	/**
	 * Gets the PrefetchBuffer associated with the TupleCollection.
	 * @return The corresponding PrefetchBuffer.
	 */
	PrefetchBuffer getPrefetchBuffer();
}