/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.api;

import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;

import java.io.Serializable;

/**
 * This class defines an object sent to the replicas to ensure durability.
 * It extends Serializable.
 * @author yoann
 */
public interface PSerializable extends Serializable {
	
	/**
	 * 
	 * @return the decoded PName of the object sent to the replicas
	 */
	PName getPName(PBinder pnc) throws PExceptionNaming;
	
	/**
	 * Set the encoded PName of the object sent to the replicas
	 * @param pName
	 */
	String setPName(PName pName);
	
	/**
	 * Get the name of the jorm class.
	 * @return
	 */
	String getClassName();
	
	/**
	 * Read all the fields of the serializable object and put it in the PAccessor pa.
	 * @param pa
	 */
	void read(PClassMapping pcm, PAccessor pa) throws PException;
	
	/**
	 * Write the PAccessor in the serializable object.
	 * 
	 */
	void write(PClassMapping pcm, PAccessor pa) throws PException;
}
