/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.facility.naming.generator;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PClassMapping;

/**
 * Provides a generator of long identifiers and ensures that allocated ones are
 * never proposed again (allocation information is made persistent).
 * @author	P. Dechamboux
 */
public interface LongGen {
	/**
	/**
	 * Generates a new long identifier.
	 * @return	The new identifier.
	 */
	long genId() throws PException;

	long genId(Object conn) throws PException;
	
	/**
	 * Allocate a range of ids.
	 * @param additional
	 * @return the last identifier allocated
	 * @throws PException
	 */
	long allocateIds(int additional) throws PException;
	
	void setIncrement(int incr);
	
	int getIncrement();
	
	/**
	 * return the id that will be assigned next time.
	 */
	long current();
}
