/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.facility.naming.generator;

import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PException;

import java.util.HashMap;

/**
 * Statically registers LongGenMgr associated with mappers within a JVM context.
 * @author	P. Dechamboux
 */
public class LongGenMgrRegistry {
	static HashMap mgrs = new HashMap();

	/**
	 * Removes this LongGenMgr from this registry.
	 */
	public static void deregisterLonGenMgr(LongGenMgr lgm) {
		mgrs.remove(lgm.getPMapper());
	}

	/**
	 * Looks for a LonGenMgr associated with this mapper.
	 */
	public static LongGenMgr getLongGenMgr(PMapper m) {
		LongGenMgr res;
		res = (LongGenMgr) mgrs.get(m);
		return res;
	}

	/**
	 * Adds this LongGenMgr from this registry.
	 */
	public static void registerLonGenMgr(LongGenMgr lgm)
			throws PException {
		if (mgrs.get(lgm.getPMapper()) != null) {
			throw new PException("LongGenMgr already registered.");
		}
		registerLonGenMgr(lgm, lgm.getPMapper());
	}
	public static synchronized void registerLonGenMgr(LongGenMgr lgm, PMapper mapper) {
		mgrs.put(mapper, lgm);
	}
}
