/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.facility.naming.olongid;

import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.facility.naming.generator.LongGenMgr;
import org.objectweb.jorm.facility.naming.generator.PLongGen;

/**
 * This object long identifier generates identifier divided in two parts. The
 * first part is the value representing a class persistent class. The second
 * part is a value representing the instance of the class. In fact this
 * generator adds a fixed value (class id) with another long values choosen
 * by a sub PLongGen.
 *
 * @author S.Chassande-Barrioz
 */
public class CompositePLongGen implements PLongGen {

	/**
	 * The inner generator of object identifier
	 */
	protected PLongGen objectIdGenerator;

	/**
	 * The identifier of the persitent class
	 */
	protected long cid;
	protected int cidSize;

	public CompositePLongGen(PLongGen objectIdGenerator, long cid, int cidSize) {
		this.objectIdGenerator = objectIdGenerator;
		this.cid = cid;
		this.cidSize = cidSize;
	}

	public long getCid() {
		return cid;
	}

	// IMPLEMENTATION OF THE PLongGen INTERFACE //
	//------------------------------------------//

	/**
	 * Initializes the underlying object identifier generator
	 */
	public void init(String name, PClassMapping pcm, LongGenMgr mgr) throws PException {
		objectIdGenerator.init(name, pcm, mgr);
	}

	/**
	 * Allocates a new composite identifier
	 */
	public long genId() throws PException {
		return genId(null);
	}

	/**
	 * Allocates a new composite identifier
	 * @param conn is a way to access the data support
	 */
	public long genId(Object conn) throws PException {
		long oid;
		if (conn != null) {
			oid = objectIdGenerator.genId(conn);
		} else {
			oid = objectIdGenerator.genId();
		}
		return (cid << (LongIdManager.LONG_SIZE - cidSize)) + oid;
	}
	
	public long allocateIds(int additional) throws PException {
		//TODO
		return -1;
	}
	public void setIncrement(int incr) {
		//TODO
	}
	
	public int getIncrement() {
		//TODO
		return -1;
	}	
	
	public long current() {
		//TODO
		return -1;
	}
}
