/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.facility.naming.rdbsequence;

import org.objectweb.jorm.facility.naming.generator.LongGen;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;

import java.sql.Connection;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class RdbSequenceLongGen
	extends RdbSequenceHelper
	implements LongGen {

	private PMapper mapper;

	public RdbSequenceLongGen(PMapperRdb mapper, String seqName, boolean sequenceCreated) {
		super(mapper.getRdbAdapter(), seqName, sequenceCreated);
		this.mapper = mapper;
	}

	public long genId() throws PException {
		Object conn = mapper.getConnection();
		try {
			return genId(conn);
		} finally {
			mapper.closeConnection(conn);
		}
	}

	public long genId(Object _conn) throws PException {
		Connection conn = RdbConnectionWrapper.narrow2SQL(_conn);
		return allocateId(conn);
	}
	
	public long allocateIds(int additional) throws PException {
		//the number of ids to allocate must be > 0
		if (additional < 1) {
			throw new PException("You cannot allocate " + additional + 
					"ids. The number of ids to allocateMust be > 0.");
		}
		if (additional > 1) {
			if(!allocateHint(mapper.getConnection(), additional)) {
				throw new PException("Problem while trying to send a first query to simulate" 
						+ " allocation of " + additional + "ids.");
			}
		}
		return genId();
	}
	
	public void setIncrement(int incr) {
		setSequenceIncrement(new Integer(incr));
	}
	
	public int getIncrement() {
		return getSequenceIncrement().intValue();
	}
	
	public long current() {
		//TODO implement it in the rdb sequence helper
		return -1;
	}
}
