/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Authors: S.Chassande-Barrioz.
 * Created on 12 juil. 2005
 *
 */
package org.objectweb.jorm.generator.lib;

import java.util.Collection;
import java.util.Iterator;

import org.objectweb.asm.ClassVisitor;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.type.api.PType;

/**
 * This class is a generation context for byte enhancement with ASM. This class
 * specializes the GenContext for the generation of code around the a Composite
 * Name (JORM Meta object). All fields are final and dedicated to the given
 * Composite Name.
 *
 * @author S.Chassande-Barrioz
 */
public class CNGenContext extends GenContext {

    /**
     * The name of the generated PBinder
     */
    public final String xbinderJCN;

    /**
     * The name of the generated PName
     */
    public final String xpnameJCN;

    /**
     * The name of the generated PNameGetter
     */
    public final String xpngJCN;
    
    /**
     * The type of the generated PBinder
     */
    public final String xbinderJT;
    
    /**
     * The type of the generated PName
     */
    public final String xpnameJT;
    
    /**
     * The type of the generated PNameGetter
     */
    public final String xpngJT;

    public final String xconstructorFieldsPnc;

    /**
     * Is the JORM meta object representing the composite name
     */
    public final CompositeName co;
    
    public final Collection coAllFields;
        
    /**
     * Indicates if all field of the composite name are calculated (the field
     * value is computed or generated, ex sequence).
     */
    public final boolean allAutoCalculated;
    
    /**
     * Indicates if all field of the composite name are not calculated (the field
     * value is computed or generated, ex sequence).
     */
    public final boolean hasAutoCalculatedField;
    
    public final PrimitiveElement uniqueField;
    
    public final boolean uniqueFieldIsString;
    public CNGenContext(ClassVisitor _clav, CommonJVMHelper h) {
        super(_clav, h);
        this.co = null;
        coAllFields = null;
        this.xbinderJCN = null;
        this.xbinderJT = null;
        this.xpnameJCN = null;
        this.xpnameJT = null;
        this.xpngJCN = null;
        this.xpngJT = null;
        this.xconstructorFieldsPnc = null;
		this.allAutoCalculated = false;
		this.hasAutoCalculatedField = false;
	    this.uniqueField = null;
	    this.uniqueFieldIsString = false;
    }
    
    public CNGenContext(CompositeName _co, ClassVisitor _clav, CommonJVMHelper h) {
        super(_clav, h);
        this.co = _co;
        coAllFields = co.getAllFields();
        this.xbinderJCN = helper.getJVMClassName(co.getFQName() + "Binder");
        this.xbinderJT = helper.getJVMType(co.getFQName() + "Binder");
        this.xpnameJCN = helper.getJVMClassName(co.getFQName() + "PName");
        this.xpnameJT = helper.getJVMType(co.getFQName() + "PName");
        this.xpngJCN = helper.getJVMClassName(co.getFQName() + "PNG");
        this.xpngJT = helper.getJVMType(co.getFQName() + "PNG");
        this.xconstructorFieldsPnc = helper.getConstructorFieldsPncParam(co);
	    
        //search auto calculated fields
        boolean aacf = true; //All Auto Calculated Field
	    boolean hacf = false; // Has Auto Calculated Field
		for (Iterator it = co.getAllFields().iterator(); it.hasNext();) {
		    boolean ac = ((PrimitiveElement) it.next()).isAutoCalculated();
		    aacf &= ac;
		    hacf |= ac;
		}
		this.allAutoCalculated = aacf;
		this.hasAutoCalculatedField = hacf;
		
		// search unique field
	    Iterator it = coAllFields.iterator();
	    PrimitiveElement pe = (PrimitiveElement) it.next();
	    if (it.hasNext()) {
	        //there are several fields
		    this.uniqueField = null;
		    this.uniqueFieldIsString = false;
	    } else {
	        //there is an unique field
	        this.uniqueField = pe;
	        this.uniqueFieldIsString = 
	            uniqueField.getType().getTypeCode() == PType.TYPECODE_STRING;
	    }

    }
}
