/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2005 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: jorm@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.jorm.generator.lib;

import org.objectweb.asm.ClassVisitor;
import org.objectweb.util.monolog.api.Logger;

import java.util.Set;

/**
 * This visitor adds a list of interface to a class.
 *
 * @author S.Chassande-Barrioz
 */
public class ClassInterfaceAdder extends LoggedClassAdapter {

    private Set itfs;

    public ClassInterfaceAdder(
            ClassVisitor classVisitor, Set itfs,Logger logger) {
        super(classVisitor, logger);
        this.itfs = itfs;
    }

    public void visit(final int version, final int access,
                      final String name,
                      final String superName,
                      final String[] interfaces,
                      final String sourceFile) {
        // just stores the interfaces implemented by the visited class
        if (interfaces != null) {
            for (int i = 0; i < interfaces.length; ++i) {
                itfs.add(interfaces[i]);
            }
        }
        String[] newInterfaces = (String[]) itfs.toArray(new String[itfs.size()]);
        cv.visit(version, access, name, superName, newInterfaces, sourceFile);
    }
}
