/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Authors: S.Chassande-Barrioz.
 * Created on 12 juil. 2005
 *
 */
package org.objectweb.jorm.generator.lib;

import org.objectweb.asm.ClassVisitor;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;

/**
 * This class defines a generation context based on byte code enhancement 
 * with ASM. In order to be efficient all fields are final.
 *
 * @author S.Chassande-Barrioz
 */
public class GenContext {

    public final String pcmJT;
    public final String objectJT;
    public final String pnameJT;
    public final String pnameJCN;
    public final String pnameManagerJT;
    public final String pngJCN;
    public final String pexceptionJCN;
    public final String stringBufferJT;
    public final String stringBufferJCN;
    public final String stringJCN;
    public final String stringJT;
    public final String ptypeJCN;
    public final String ptypeJT;
    public final String ptypespaceJCN;
    public final String ptypespaceJT;

    /**
     * The visitor used to generate the class
     */
    public final ClassVisitor clav;
    public final CommonJVMHelper helper;
    
    public GenContext(ClassVisitor _clav, CommonJVMHelper h) {
        this.clav = _clav;
        this.helper = h;
        pcmJT = helper.getJVMType(PClassMapping.class);
        objectJT = helper.getJVMType(Object.class);
        pnameJT = helper.getJVMType(PName.class);
        pnameJCN = helper.getJVMClassName(PName.class);
        pnameManagerJT = helper.getJVMType(PNameManager.class);
        pngJCN = helper.getJVMClassName(PNameGetter.class);
        pexceptionJCN = helper.getJVMClassName(PException.class);
        stringBufferJT = helper.getJVMType(StringBuffer.class);
        stringBufferJCN = helper.getJVMClassName(StringBuffer.class);
        stringJCN = helper.getJVMClassName(String.class);
        stringJT = helper.getJVMType(String.class);
        ptypeJCN = helper.getJVMClassName(PType.class);
        ptypeJT = helper.getJVMType(PType.class);
        ptypespaceJCN = helper.getJVMClassName(PTypeSpace.class);
        ptypespaceJT = helper.getJVMType(PTypeSpace.class);
    }
}
