/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.generator.lib;

import org.objectweb.asm.ClassWriter;
import org.objectweb.jorm.compiler.api.PExceptionCompiler;
import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.jorm.metainfo.api.*;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.jorm.util.lib.StringReplace;
import org.objectweb.util.monolog.api.BasicLevel;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Iterator;

/**
 * This class is a generator of XXXPNG objects. It is generic either the mapper
 * type. This generator use the velocity tools. The used template is PNG.vm .
 * Associated to this template this generator builds a velocity context which
 * contains the following information: <br/><table border=1>
 * <tr>
 * <td>Key</td>
 * <td>Value</td>
 * </tr>
 * <tr>
 * <td>"compositename"</td>
 * <td>The reference to the compositename meta object</td>
 * </tr>
 * <tr>
 * <td>"tools"</td>
 * <td>The reference to the <a href="CommonHelper.html">CommonHelper <a/></td>
 * </tr>
 * <tr>
 * <td>"header"</td>
 * <td>The file name of the common template which contains the header of the
 * generate files.</td>
 * </tr>
 * </table>
 */
public class PNameGetterGenerator extends CommonGenerator {
    /**
     * This method generates a XPNameGetter file corresponding to the mo
     * parameter in the directory parameter.
     * 
     * @param cn
     *            The meta object interface.
     * @param holder
     *            The target holder which allows to create files
     * @param cp
     *            This parameter permits to reach the compilation parameters.
     */
    public void generate(CompositeName cn, TargetHolder holder,
            JormCompilerParameter cp) throws PException {
        boolean debug = logger != null && logger.isLoggable(BasicLevel.DEBUG);
        ClassWriter cw = new ClassWriter(true);
        String itfname = getPNGItfName(cn.getFQName());
        if (debug) {
            logger.log(BasicLevel.DEBUG, "Interface " + itfname);
        }
        cw.visit(V1_1, ACC_PUBLIC + ACC_ABSTRACT + ACC_INTERFACE, //access
                itfname,
                getJVMClassName(Object.class),
                null,
                itfname);
        for (Iterator fieldIt = cn.getAllFields().iterator(); fieldIt.hasNext();) {
            TypedElement te = (TypedElement) fieldIt.next();
            String fn = upperFL(te.getName());
            cw.visitMethod(ACC_PUBLIC + ACC_ABSTRACT, 
                    "pnGet" + fn, 
                    "(" + getJVMType(Object.class) + ")" + getJVMType(te.getType()),
                    new String[] {getJVMClassName(PException.class) }, 
                    null);
        }
        try {
            // Fetch a block with the fileName
            String fileName = itfname;
            fileName = StringReplace.replaceChar('/', File.separatorChar,
                    fileName)
                    + ".class";
            FileOutputStream fos = holder.getFileOutputStream(fileName);
            fos.write(cw.toByteArray());
            fos.close();
        } catch (Exception e) {
            throw new PExceptionCompiler(e,
                    "Problem while generating PAccessor.");
        }
    }
    public String getPNGItfName(String className) {
		return getJVMClassName(className + "PNG");
    }    

}