/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.lib;

import java.io.Serializable;

import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.naming.api.PName;

/**
 * This class is the default implementation of the PBinding interface. 
 *  
 * @author S.Chassande-Barrioz
 */
public class PBindingImpl implements PBinding, PBindingCtrl, Serializable {

	transient protected byte status;

	transient public PClassMapping pClassMapping;
	
	transient protected PName pname;
	

	public PBindingImpl() {
	}
	
	public PBindingImpl(PClassMapping pcm) {
		this.pClassMapping = pcm;
		status = PBinding.LIFECYCLE_NOTBOUND;
	}

	public void bind(PName pn) throws PException {
		if (pname!=null) {
			// The Pbinding must be unbinded only if the PName has been already
			// defined
			pClassMapping.getPBinder().unbind(this);
		}
		pClassMapping.getPBinder().bind(pn, this);
	}

	public boolean exist(Object conn) throws PException {
		return pClassMapping.exist(this, conn);
	}

	public PName export(Object conn) throws PException {
		return pClassMapping.getPBinder().export(conn, this);
	}

	public PName export(Object conn, Object hints) throws PException {
		return pClassMapping.getPBinder().export(conn, this, hints);
	}

	public PClassMapping getPClassMapping() {
		return pClassMapping;
	}

	public PName getPName() {
		return pname;
	}

	public byte getStatus() {
		return status;
	}

	public void setPName(PName pn) {
		this.pname = pn;
	}

	public void setStatus(byte s) {
		this.status = s;
	}

	public void init(PClassMapping pcm) throws PException {
		this.pClassMapping = pcm;
		this.status = PBinding.LIFECYCLE_NOTBOUND;
	}

	public void read(Object conn, PAccessor pa) throws PException {
		if (pClassMapping == null) {
			throw new PExceptionProtocol("PBinding is not initialized");
		}
		pClassMapping.read(this, conn, pa);
	}

    public void read(Object conn, PAccessor pa, Object txctx) throws PException {
        if (pClassMapping == null) {
            throw new PExceptionProtocol("PBinding is not initialized");
        }
        pClassMapping.read(this, conn, pa, txctx);
    }

    public void read(Object conn, PAccessor pa, Object txctx, boolean forUpdate) throws PException {
        if (pClassMapping == null) {
            throw new PExceptionProtocol("PBinding is not initialized");
        }
        pClassMapping.read(this, conn, pa, txctx, forUpdate);
    }

	public void unbind() throws PException {
		if (pClassMapping == null) {
			throw new PExceptionProtocol("PBinding is not initialized");
		}
		pClassMapping.getPBinder().unbind(this);
	}

	public void unexport(Object conn) throws PException {
		if (pClassMapping == null) {
			throw new PExceptionProtocol("PBinding is not initialized");
		}
		pClassMapping.getPBinder().unexport(conn, pname, this);
	}

	public void write(Object conn, PAccessor pa) throws PException {
		if (pClassMapping == null) {
			throw new PExceptionProtocol("PBinding is not initialized");
		}
		pClassMapping.write(this, conn, pa);
	}
}
