/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Authors: S.Chassande-Barrioz.
 * Created on 4 juil. 2005
 *
 */
package org.objectweb.jorm.lib;

import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PSerializable;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;

/**
 * This class is incomplete implementation of PSerializable interface. This
 * class is subclasses by generated XXXserializable class.
 *
 * @author S.Chassande-Barrioz
 */
public abstract class PSerializableImpl implements PSerializable, PAccessor {

	protected String pName;


    public abstract String getClassName();

    public abstract void read(PClassMapping pcm, PAccessor pa) throws PException;

    public abstract void write(PClassMapping pcm, PAccessor pa) throws PException;

	public Object getMemoryInstance(){
		return this;
	}

    public PName getPName(PBinder pnc) throws PExceptionNaming {
		try{
			return pnc.decodeString(pName);
		}catch (Exception e) {
              throw new PExceptionNaming("Cannot decode to a PName: " + e.getMessage());
        }
    }

    public String setPName(PName pName) {
		if(pName.codingSupported((short) PNameCoder.CTSTRING)){
			try{
				this.pName  = pName.encodeString();
			} catch(Exception e){
			}
		} else{
			this.pName = "null";
		}
		return this.pName;
    }
    
    /**
     * Encodes a PName into a String value
     * @param pn is the PName to encode
     * @return a String representation of the PName
     */
    protected String encodeString(PName pn) {
		if(pn.codingSupported((short) PNameCoder.CTSTRING)){
			try {
				return pn.encodeString();
			}
			catch(PException e){
			}
		}
		return "null";
    }
    
    /**
     * Decodes a String value in a PName. This value is linked to a persistent
     * field.
     * @param val is a String representation of a PName
     * @param fieldName is the name of the field associated to the reference
     * @param hints has to be PClassMapping or a PNameCoder
     * @return a PName instance or null.
     */
	protected PName decodeString(String val, String fieldName, Object hints){
		PNameCoder pnc;
		if(hints instanceof PClassMapping){
			pnc = ((PClassMapping) hints).getPNameCoder(fieldName);
		} else if(hints instanceof PNameCoder){
			pnc = (PNameCoder) hints;
		} else {
			return null;
		}
		try{
			return pnc.decodeString(val);
		} catch(PException e){
			return null;
		}
	}
    
}
