/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.fos.metainfo;

import org.objectweb.jorm.metainfo.lib.BasicMapping;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.ClassMapping;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * @author  P. Dechamboux
 */
public class FosMapping extends BasicMapping {
	/**
	 * Builds a new BasicRdbMapping object.
	 * This object contains the mapping structures of a class.
	 * The parent object is a Project object.
	 * @param mapperName   the mapper name,
	 *        parent	    the parent of the current object.
	 */
	public FosMapping(String mapperName, MetaObject parent) {
		super(mapperName, parent);
	}

	/**
	 * Creates a new BasicRdbClassMapping object.
	 * @param   ruleName   the name of the rule used to map the class,
	 *          linkedMO   the Class object referenced by the current object,
	 *          parent     the parent object of the current object.
	 * @return	an RdbClassMapping object.
	 */
	public ClassMapping createClassMapping(String ruleName) {
		FosClassMapping res = (FosClassMapping) getClassMapping();
		if (res == null) {
			res = new FosClassMapping(getParent().getParent(), this);
			setClassMapping(res);
			res.setLogger(logger);
		} else {
			res.setRuleName(ruleName);
		}
		return res;
	}

	/**
	 * Creates a new BasicFosGenClassMapping object.
	 * @param   ruleName   the name of the rule used to map the class,
	 *          linkedMO   the GenClassRef object referenced by the current
	 *                     object,
	 *          parent     the parent object of the current object.
	 * @return	an BasicFosGenClassMapping object.
	 */
	public FosGenClassMapping createGenClassMapping(String ruleName,
														 MetaObject linkedMO) {
		String gcid = ((GenClassRef) linkedMO).getGenClassId();
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, "(gcid " + gcid + ")");
		}
		FosGenClassMapping res = (FosGenClassMapping) getGenClassMapping(gcid);
		if (res == null) {
			res = new FosGenClassMapping(linkedMO, this);
			res.setLogger(logger);
			addGenClassMapping(gcid, res);
		} else {
			res.setRuleName(ruleName);
			res.setLinkedMO(linkedMO);
		}
		return res;
	}
}
