/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.genclass;

import org.objectweb.jorm.genclass.api.FieldDesc;
import org.objectweb.jorm.type.api.PType;

/**
 * This class provides a description of a JORM field to be mapped to a RDBMS.
 * It allows the association of a column name to a field.
 * @author	P. Dechamboux
 */

public class RdbFieldDesc extends FieldDesc {
	/**
	 * The column name to which the JORM field should be mapped.
	 */
	public String columnName;

	/**
	 * The column type to which the JORM field should be mapped.
	 */
	public String columnType;

	/**
	 * This field maps to a column that belongs to another table. It completes
	 * a reference element.
	 */
	public boolean isRefExtField = false;

	/**
	 * It constructs an RdbFieldDesc.
	 */
	public RdbFieldDesc(String fn, String cfn, PType ft, String cn, String ct) {
		super(fn, cfn, ft);
		columnName = cn;
		columnType = ct;
	}

	/**
	 * It constructs an RdbFieldDesc.
	 */
	public RdbFieldDesc(String fn, String cfn, PType ft, String cn, String ct, int sz, int scale) {
		super(fn, cfn, ft, sz, scale);
		columnName = cn;
		columnType = ct;
	}

	/**
	 * It constructs an RdbFieldDesc.
	 */
	public RdbFieldDesc(String fn, String cfn, PType ft, String cn, String ct, boolean isRefExtField) {
		this(fn, cfn, ft, cn, ct);
		this.isRefExtField = isRefExtField;
	}

	/**
	 * It constructs an RdbFieldDesc.
	 */
	public RdbFieldDesc(String fn, String cfn, PType ft, String cn, String ct, int sz, int scale, boolean isRefExtField) {
		this(fn, cfn, ft, cn, ct, sz, scale);
		this.isRefExtField = isRefExtField;
	}
}
