/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.lib;

import java.io.FileWriter;
import java.io.IOException;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;

/**
 * This class is used to define a dummy PMappingStructuresManager
 * to generate the database creation script.
 * @author Y.Bersihand
 */
public class RdbScriptPMSM extends RdbPMappingStructuresManager implements
		RdbScriptWriter {

	private boolean writeScript = false;
	private String fileName;
	private FileWriter fileWriter = null;
	
	public RdbScriptPMSM(String fileName, boolean writeScript) {
		this.fileName = fileName;
		this.writeScript = writeScript;
	}
	
	/**
	 * Creates the file.
	 * The file name MUST be specified before calling this method.
	 */
	public void init() throws PException {
		if (fileName == null) {
			throw new PException("A file name must be specified before calling the init method.");
		}
		if (writeScript) {
			try {
				//create the file: it removes it if it already exists
				fileWriter = new FileWriter(fileName, false);
				fileWriter.write("");
				fileWriter.close();
				//then keep a reference to it to append text in it
				fileWriter = new FileWriter(fileName, true);
			} catch (IOException ioe) {
				throw new PExceptionIO(ioe, "Problem while creating the database creation script file.");
			}
		}
	}
	
	public String getFileName() {
		return fileName;
	}
	public void setFileName(String fileName) {
		this.fileName = fileName;
	}
	
	public FileWriter getFileWriter() {
		return fileWriter;
	}
	
	public boolean isWriteScript() {
		return writeScript;
	}
	public void setWriteScript(boolean writeScript) {
		this.writeScript = writeScript;
	}

}
