/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.metainfo.api.MappingStructure;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.metainfo.api.ClassProject;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.jorm.util.api.Loggable;

/**
 * Description
 */
public class BasicMappingStructure
    extends BasicMetaObject
    implements MappingStructure {

	/**
	 * The MetaObject object referenced by the current object.
	 */
	protected MetaObject linkedMO;

	/**
	 * Builds a new BasicMappingStructure object.
	 * @param linkedMO  the MetaObject object referenced by the current object.
	 */
	public BasicMappingStructure(MetaObject parent, MetaObject linkedMO) {
        super(parent);
        this.linkedMO = linkedMO;
    }

	///////////////////////////////////////////////////////////////////
	//                      from MappingStructure interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Returns the MetaObject referenced by the current MetaObject.
	 * @return	the MetaObject referenced by the current object. If there is
     *          no referenced object, null is returned.
	 */
    public MetaObject getLinkedMO() {
        return linkedMO;
    }

	/**
	 * Sets the MetaObject referenced by the current MetaObject.
	 * @param linkedMO	  the MetaObject referenced by the current object
     */
    public void setLinkedMO(MetaObject linkedMO) {
        this.linkedMO = linkedMO;
    }

	/**
	 * Returns the mapper name.
	 * @return the mapper name.
	 */
    public String getMapperName() {
		MetaObject mo = this;
		while (!(mo instanceof Mapping)) {
			mo = mo.getParent();
		}
        return (mo != null
                ? ((Mapping) mo).getMapperName()
                : "");
    }

	/**
	 * Returns the project name.
	 * @return the project name.
	 */
    public String getProjectName() {
		MetaObject mo = getParent();
		while (! (mo instanceof ClassProject)) {
			mo = mo.getParent();
		}
        return  (mo != null
                ? ((ClassProject) mo).getProjectName()
                : "");
    }
}
