/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.ParentClassMapping;
import org.objectweb.jorm.metainfo.api.Class;

public class BasicParentClassMapping
    extends BasicMappingStructure
    implements ParentClassMapping {

	/**
	 * The name of the rule involved in the mapping of the inheritance relationship.
     */
	private String ruleName;

	/**
	 * Builds a new BasicReferenceMapping object.
	 * @param ruleName    the rule name
	 * @param superClass  the super class in the inheritance hierarchy
	 * @param parentMO    the parent meta-object in the meta-object tree
	 */
	public BasicParentClassMapping(String ruleName, Class superClass, MetaObject parentMO) {
		super(parentMO, superClass);
		this.ruleName = ruleName;
	}

	///////////////////////////////////////////////////////////////////
	//                      from ParentClassMapping interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Returns the name of the rule involved in the mapping of the inheritance relationship.
	 * @return	 the name of the rule.
	 */
	public String getRuleName() {
		return this.ruleName;
	}

	/**
	 * Sets a rule name.
	 * @param rulename the name of the rule.
	 */
	public void setRuleName(String rulename) {
		this.ruleName = rulename;
	}

	/**
	 * Returns the Fully Qualified Name of the associated extension.
	 * @return	 the name of the rule.
	 */
	public String getFQName() {
        return ((Class) getLinkedMO()).getFQName();
    }

	/**
	 * Returns the Class object designated by the extension.
	 * @return	 a Class object.
	 */
	public Class getMOClass() {
        return (Class) getLinkedMO();
    }
}

