/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mi2xml.api;

import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.ClassMapping;
import org.objectweb.jorm.metainfo.api.GenClassMapping;
import org.objectweb.jorm.api.PException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.util.Map;

/**
 * MappingDomtreeBuilder builds a DOMTree representation of a class mapping specification
 * or a generic class mapping specification.
 */
public interface MappingDomtreeBuilder {

	/**
	 * Assigns the current Class object to the MappingDomtreeBuilder object.
	 *
	 * @param currentClass	the Jorm meta-object associated to the parsed class.
	 */
	void setCurrentClass(Class currentClass);

	/**
	 * Assigns the current Class object to the MappingDomtreeBuilder object.
	 *
	 * @param metaobject2idvalue	an HashTable that associates a MetaObject with an id value.
	 */
	void setmetaobject2idvalue(Map metaobject2idvalue);

	/**
	 * Adds a class mapping element to a mapping element.
	 * @param   document        an org.w3c.dom.Document object,
	 *          mappingElement  a mapping element,
	 *          classMapping    a <tt>ClassMapping</tt> object.
	 */
	abstract void processClassMapping(Document document, Element mappingElement,
									  ClassMapping classMapping) throws PException;

	/**
	 * Adds a generic class mapping element to a mapping element.
	 * @param   document            an org.w3c.dom.Document object,
	 *          mappingElement      a mapping element,
	 *          genClassMapping     a <tt>GenClassMapping</tt> object.
	 */
	abstract void processGenClassMapping(Document document, Element mappingElement,
										 GenClassMapping genClassMapping) throws PException;
}
