/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.naming.lib;

import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.api.PException;

import java.util.Date;
import java.util.Map;
import java.util.HashMap;
import java.math.BigInteger;
import java.math.BigDecimal;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class MapPNameGetter implements PNameGetter {

	Map field2value;

	public MapPNameGetter(String[] fieldNames, Object[] fieldValues) {
		field2value = new HashMap();
		for(int i=fieldNames.length-1; i>=0; i--) {
			field2value.put(fieldNames[i], fieldValues[i]);
		}
	}
	public MapPNameGetter(Map field2value) {
		this.field2value = field2value;
	}

	public MapPNameGetter() {
		field2value = new HashMap();
	}

	public void set(String fn, byte b) {
		field2value.put(fn, new Byte(b));
	}

	public void set(String fn, char c) {
		field2value.put(fn, new Character(c));
	}

	public void set(String fn, short s) {
		field2value.put(fn, new Short(s));
	}

	public void set(String fn, int n) {
		field2value.put(fn, new Integer(n));
	}

	public void set(String fn, long l) {
		field2value.put(fn, new Long(l));
	}

	public void set(String fn, Object o) {
		field2value.put(fn, o);
	}

	public byte pngetByteField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
		}
		if (value instanceof Byte) {
			return ((Byte) value).byteValue();
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public Byte pngetObyteField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			if (field2value.containsKey(fn)) {
				return null;
			} else {
				throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
			}
		}
		if (value instanceof Byte) {
			return (Byte) value;
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public char pngetCharField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
		}
		if (value instanceof Character) {
			return ((Character) value).charValue();
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public Character pngetOcharField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			if (field2value.containsKey(fn)) {
				return null;
			} else {
				throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
			}
		}
		if (value instanceof Character) {
			return (Character) value;
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public short pngetShortField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
		}
		if (value instanceof Short) {
			return ((Short) value).shortValue();
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public Short pngetOshortField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			if (field2value.containsKey(fn)) {
				return null;
			} else {
				throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
			}
		}
		if (value instanceof Short) {
			return (Short) value;
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public int pngetIntField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
		}
		if (value instanceof Integer) {
			return ((Integer) value).intValue();
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public Integer pngetOintField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			if (field2value.containsKey(fn)) {
				return null;
			} else {
				throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
			}
		}
		if (value instanceof Integer) {
			return (Integer) value;
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public long pngetLongField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
		}
		if (value instanceof Long) {
			return ((Long) value).longValue();
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public Long pngetOlongField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			if (field2value.containsKey(fn)) {
				return null;
			} else {
				throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
			}
		}
		if (value instanceof Long) {
			return (Long) value;
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public String pngetStringField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			if (field2value.containsKey(fn)) {
				return null;
			} else {
				throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
			}
		}
		if (value instanceof String) {
			return (String) value;
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public byte[] pngetByteArrayField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			if (field2value.containsKey(fn)) {
				return null;
			} else {
				throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
			}
		}
		if (value instanceof byte[]) {
			return (byte[]) value;
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public char[] pngetCharArrayField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			if (field2value.containsKey(fn)) {
				return null;
			} else {
				throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
			}
		}
		if (value instanceof char[]) {
			return (char[]) value;
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public Date pngetDateField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			if (field2value.containsKey(fn)) {
				return null;
			} else {
				throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
			}
		}
		if (value instanceof Date) {
			return (Date) value;
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public BigInteger pngetBigIntegerField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			if (field2value.containsKey(fn)) {
				return null;
			} else {
				throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
			}
		}
		if (value instanceof BigInteger) {
			return (BigInteger) value;
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}

	public BigDecimal pngetBigDecimalField(String fn, Object ctxt) throws PException {
		Object value = field2value.get(fn);
		if (value == null) {
			if (field2value.containsKey(fn)) {
				return null;
			} else {
				throw new PException("No field " + fn + " found among "
				+ field2value.keySet());
			}
		}
		if (value instanceof BigDecimal) {
			return (BigDecimal) value;
		}
		throw new PException("Bad type: expected " + value.getClass().getName());
	}
}
