/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.naming.lib;

import org.objectweb.jorm.naming.api.PNameGetterConverter;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.api.PException;

import java.util.Date;
import java.math.BigInteger;
import java.math.BigDecimal;

/**
 * is a simple PNameGetterConverter which does not make any translation
 *
 * @author S.Chassande-Barrioz
 */
public class SimplePNGConverter implements PNameGetterConverter {

	private class MyPNG implements PNameGetter {
		Object value;

		public MyPNG(Object o) {
			this.value = o;
		}

		public byte pngetByteField(String fn, Object ctxt) throws PException {
			return ((Byte) value).byteValue();
		}

		public Byte pngetObyteField(String fn, Object ctxt) throws PException {
			return (Byte) value;
		}

		public char pngetCharField(String fn, Object ctxt) throws PException {
			return ((Character) value).charValue();
		}

		public Character pngetOcharField(String fn, Object ctxt) throws PException {
			return (Character) value;
		}

		public short pngetShortField(String fn, Object ctxt) throws PException {
			return ((Short) value).shortValue();
		}

		public Short pngetOshortField(String fn, Object ctxt) throws PException {
			return (Short) value;
		}

		public int pngetIntField(String fn, Object ctxt) throws PException {
			return ((Integer) value).intValue();
		}

		public Integer pngetOintField(String fn, Object ctxt) throws PException {
			return (Integer) value;
		}

		public long pngetLongField(String fn, Object ctxt) throws PException {
			return ((Long) value).longValue();
		}

		public Long pngetOlongField(String fn, Object ctxt) throws PException {
			return (Long) value;
		}

		public String pngetStringField(String fn, Object ctxt) throws PException {
			return (String) value;
		}

		public byte[] pngetByteArrayField(String fn, Object ctxt) throws PException {
			return (byte[]) value;
		}

		public char[] pngetCharArrayField(String fn, Object ctxt) throws PException {
			return (char[]) value;
		}

		public Date pngetDateField(String fn, Object ctxt) throws PException {
			return (Date) value;
		}

		public BigInteger pngetBigIntegerField(String fn, Object ctxt) throws PException {
			return (BigInteger) value;
		}

		public BigDecimal pngetBigDecimalField(String fn, Object ctxt) throws PException {
			return (BigDecimal) value;
		}
	}

	public PNameGetter convert(PNameGetter png) throws PExceptionNaming {
		return png;
	}

	public PNameGetter convert(Object o) throws PExceptionNaming {
		return new MyPNG(o);
	}

	public PNameGetter convert(byte o) throws PExceptionNaming {
		return new MyPNG(new Byte(o));
	}

	public PNameGetter convert(char o) throws PExceptionNaming {
		return new MyPNG(new Character(o));
	}

	public PNameGetter convert(short o) throws PExceptionNaming {
		return new MyPNG(new Short(o));
	}

	public PNameGetter convert(int o) throws PExceptionNaming {
		return new MyPNG(new Integer(o));
	}

	public PNameGetter convert(long o) throws PExceptionNaming {
		return new MyPNG(new Long(o));
	}
}
