/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.util.io.api;

import java.util.Collection;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * PathExplorer interface defines methods to create object able
 * to manage a list of paths and furnish input and output stream
 * on files
 */
public interface PathExplorer {

	/**
	 * adds a classpath and its path separator to the current explorer.
	 * This classpath is added to the list of path already defines.
	 * @param cpath	the classpath which contains a list of paths
	 */
	void addPath(String cpath);

	/**
	 * creates the paths with the given path
	 * @param path	the vector of paths (which are String).
	 */
	void addPath(Collection path);

	/**
	 * gets an input stream from a given file. This file is opened from the
	 * defined path.
	 * @param file	the string representation of the name of the file
	 * @return	an InputStream for the given file
	 */
	InputStream getInputStream(String file);

	/**
	 * gets an output stream from a given file. This file is opened from the
	 * defined path.
	 * @param file	the string representation of the name of the file
	 * @return	an OutputStream for the given file
	 */
	OutputStream getOutputStream(String file);


	String getClassPath();

}
