/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.util.io.api;

import java.util.Iterator;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;

/**
 * A TargetHolder ...
 */
public interface TargetHolder {

	/**
	 * This method permits to obtain a FileWriter either the filename specified
	 * in parameter.
	 */
	FileWriter getFileWriter(String fileName) throws Exception;

	FileOutputStream getFileOutputStream(String fileName) throws Exception;

	File getFile(String filename);
	
	/**
	 * Return an iterator on the file managed by this holder.
	 * Element type of the returned iterator is java.io.File
	 */
	Iterator iterateFile();

	/**
	 * Return an iterator on the file writer  maneged by this holder.
	 * Element type of the returned iterator is java.io.FileWriter
	 */
	Iterator iterateFileWriter();

	/**
	 * This method compiles all files managed by this holder
	 */
	void javacAll() throws Exception;
}
