/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.util.io.lib;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;

import org.objectweb.jorm.util.io.api.TargetHolder;

/**
 * A TargetHolder ...
 */
public class JavaFileHolder implements TargetHolder {

	private Hashtable name2fw = null;

	private File targetDirectory = null;

	public JavaFileHolder(String _targetDirectory) throws Exception {
		name2fw = new Hashtable();
		if (_targetDirectory == null) {
			String currentDir = System.getProperties().getProperty("user.dir");
			targetDirectory = new File(currentDir);
		} else {
			targetDirectory = new File(_targetDirectory);
		}
		if (!targetDirectory.isDirectory())
			throw new Exception("The parameter must be a directory.");
	}

	// IMPLEMENTATION OF THE TargetHolder INTERFACE //

	/**
	 * This method permits to obtain a FileWriter either the filename specified
	 * in parameter.
	 */
	public synchronized FileWriter getFileWriter(String fileName) throws Exception {
		File child = new File(targetDirectory, fileName);
		FileWriter result = (FileWriter) name2fw.get(child);
		if (result == null) {
			// The file is not referenced into the hashtable

			File parentDir = child.getParentFile();
			if (!parentDir.exists()) {
				//The directory does'nt exist => create it
				parentDir.mkdirs();
			}

			// Create the file if necessary see java.io.file.createNewFile()
			child.createNewFile();
			result = new FileWriter(child);
			name2fw.put(child, result);
		}
		return result;
	}

	public synchronized FileOutputStream getFileOutputStream(String fileName) throws Exception {
	    
		File child = new File(targetDirectory, fileName);
		File parentDir = child.getParentFile();
		if (!parentDir.exists()) {
			//The directory does'nt exist => create it
			parentDir.mkdirs();
		}

		child.createNewFile();
		FileOutputStream fos = new FileOutputStream(child);
		return fos;
	}

	public File getFile(String filename) {
	    return new File(targetDirectory, filename);
	}
	
	/**
	 * Return an iterator on the absolute file managed by this holder.
	 * Element type of the returned iterator is java.util.File
	 */
	public synchronized Iterator iterateFile() {
		return name2fw.keySet().iterator();
	}

	/**
	 * Return an iterator on the absolute file writer managed by this holder.
	 * Element type of the returned iterator is java.util.FileWriter
	 */
	public synchronized Iterator iterateFileWriter() {
		return name2fw.entrySet().iterator();
	}

	/**
	 * This method compiles all files managed by this holder
	 */
	public synchronized void javacAll() throws Exception {
		//TODO
	}
}
