/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.jorm.util.lib;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.MatchingTask;
import org.apache.tools.ant.taskdefs.UpToDate;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapCluster;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.mapper.rdb.lib.MapperJDBC;
import org.objectweb.jorm.mapper.rdb.lib.RdbScriptPMSM;

/**
 * Generate the database creation script and write it into a file.
 * Example of a build.xml file for the invoice example:
 * <target name="generate.script" depends="compile">
 * 		<taskdef name="generate" classname="org.objectweb.jorm.util.lib.AntScriptGenerator">
 * 			<classpath refid="classpath"/>
 * 		</taskdef>
 * 		<generate destFile="dbCreation.sql" mapperName="rdb.postgres"
 * 			src="${build}"  classpathref="classpath"/>
 * </target>
 * 
 * 
 * @author Y.Bersihand
 */
public class AntScriptGenerator  extends MatchingTask {
	public static final String SCRIPT_FILE_NAME = "script.fileName";
	public static final String SCRIPT_MAPPER_NAME = "script.mapperName";
	public static final String SCRIPT_DELETE_STATEMENT = "script.generate.delete";
	public static final String SCRIPT_DROP_STATEMENT = "script.generate.drop";
	public static final String SCRIPT_CREATE_STATEMENT = "script.generate.create";
	public static final String SCRIPT_CLASS_DIR = "script.class.dir";
	public static final String SCRIPT_JAR_DIR = "script.jar.dir";
	
	//the classpath
	private Path classpath = null;
	//the base directory for classes to process 
	private File src = null;
	//the file in which the script is written
	protected File destFile = null;
	//the mapper name
	private String mapperName;
	//if true(default value), generate sql statements
	protected boolean generateDrop = true;
	protected boolean generateDelete = true;
	protected boolean generateCreate = true;
	
	private boolean userDefinedIncludes = false;
	//the dummy PMSM to write in a file
	protected RdbScriptPMSM msm = null;
	// the classes to process
	protected List classesToProcess = null;
	
	public static void main(String[] args) {
		new AntScriptGenerator().execute(args);
	}
	
	public RdbScriptPMSM getRdbScriptPMSM() {
		return msm;
	}
	
	//return a list of String
	public List getClassesToProcess() {
		return classesToProcess;
	}
	
	public void setClasspath(Path cp) {
        if (classpath == null)
            classpath = cp;
        else
            classpath.append(cp);
    }
    public Path getClasspath() {
        return classpath;
    }
    public Path createClasspath() {
        if (classpath == null)
            classpath = new Path(getProject());
        return classpath.createPath();
    }
    public void setClasspathRef(Reference r) {
        createClasspath().setRefid(r);
    }
	
    public void setSrc(File src) {
        this.src = src;
    }
    public File getSrc() {
		return src;
	}
    
    public void setDestFile(File destFile) {
        this.destFile = destFile;
    }
    public File getDestFile() {
		return destFile;
	}
    
    public void setGenerateDrop(boolean generate) {
    	this.generateDrop = generate;
    }
	public boolean isGenerateDrop() {
		return generateDrop;
	}
	
    public void setGenerateDelete(boolean generate) {
    	this.generateDelete = generate;
    }
	public boolean isGenerateDelete() {
		return generateDelete;
	}
    
    public void setGenerateCreate(boolean generate) {
    	this.generateCreate = generate;
    }
    public boolean isGenerateCreate() {
		return generateCreate;
	}
    
    public String getMapperName() {
		return mapperName;
	}
	public void setMapperName(String mapperName) {
		this.mapperName = mapperName;
	}
	
    public DirectoryScanner getDirectoryScanner() {
        return super.getDirectoryScanner(src);
    }
    
    public void setIncludes(String s) {
    	super.setIncludes(s);
    	userDefinedIncludes = true;
    }
    
    public boolean isUserDefinedIncludes() {
    	return userDefinedIncludes;
    }
    
    public void execute() {
    	try {
			//get the classes to process
			if (!userDefinedIncludes) {
				super.setIncludes("**/*Mapping.class");
			}
			classesToProcess = Arrays.asList(getDirectoryScanner().getIncludedFiles());
			if (classesToProcess == null || classesToProcess.isEmpty()) {
				log("[WARN] No class files found in " 
				        + src.getAbsolutePath() + ".", Project.MSG_WARN);
				return;
			}
	
			UpToDate u = new UpToDate();
			u.setProject(getProject());
			u.addSrcfiles(fileset);
			u.setTargetFile(destFile);
			if (u.eval()) {
			    return;
			}
	
			List tmpClasses = new ArrayList(classesToProcess.size());
			for (int i=0; i<classesToProcess.size(); i++) {
				//format the string
				tmpClasses.add(formatElement((String) classesToProcess.get(i)));
			}
			
	
			msm = new RdbScriptPMSM(destFile.getAbsolutePath(), true);
			try {
				//create a mapper
				MapperJDBC mapper = new MapperJDBC();
				//assign a name
				mapper.setMapperName(mapperName);
				//start it
				mapper.start();
				//assign it to the msm
				msm.setPMapper(mapper);
				//create the file
				msm.init();
			} catch (PException pe) {
				log(
				        "[ERROR] Impossible to init the PMappingStructuresManager :"
				        + pe.getMessage(), Project.MSG_ERR);
			}
			//get the class loader
			ClassLoader cl = getClass().getClassLoader();
	
			//make the copy
			 classesToProcess = new ArrayList(tmpClasses);
			//init all the classes
			for (int i=0; i<classesToProcess.size(); i++) {
				//if the init is not ok, 
				//then remove the class from the list to process in the second stage
				if(initClass((String)classesToProcess.get(i), msm, cl) == -1) {
					tmpClasses.remove(classesToProcess.get(i));
				}
			}
			//copy the classes to process in the second stage in the list
			classesToProcess = new ArrayList(tmpClasses);
			//keep a list of clusters already processed
			Collection clusterList = new ArrayList(1);
			//generate sql statements
			for (int i=0; i<classesToProcess.size(); i++) {
				try {
					//get the class name
					String classMapping = (String)classesToProcess.get(i);
					String className = classMapping;
					int index = classMapping.indexOf(PMapper.PCLASSMAPPINGAPPENDER);
					if (index != -1) {
						className = classMapping.substring(0, index);
					}
					//get the cluster
					PMapCluster cluster = msm.getPMapCluster(className);
					if (cluster == null) {
						log("[ERROR] JORM cluster is null for class: " + className, 
						        Project.MSG_ERR);
					}
					//to avoid writting the same statements many times
					if (clusterList.contains(cluster)) {
						continue;
					} else {
						clusterList.add(cluster);
					}
					if (!cluster.isDefined()) {
						Iterator it = cluster.getUnResolvedDependencies().iterator();
						while( it.hasNext()) {
							String classUnresolved = (String) it.next();
							if (className.equals(classUnresolved)) {
								cluster.classDefined(className);
							} else {
								log("[INFO] Cluster for class: " + className + " is not ready : " + 
										cluster.getUnResolvedDependencies().toString(), 
								        Project.MSG_INFO);
								initClass(classUnresolved + PMapper.PCLASSMAPPINGAPPENDER , msm, cl);
							}
						}
					} 
					
					if (generateDrop) {
						cluster.deleteMappingStructures();
					}
					if (generateCreate) {
						cluster.createMappingStructures(true);
					}
					if (generateDelete) {
						cluster.deleteData();
					}
				} catch(Exception pe) {
					log("[ERROR] Cannot generate sql statements : " + pe.getMessage(), 
					        Project.MSG_ERR);
				}
			}
    	} catch (Exception e) {
    		e.printStackTrace();
    	}
    }
    
	public void execute(String[] args){
        //Check parameter
		if (args.length == 0) {
			usage("You must specify classes to generate script for.");
		}
		//get the fileName
		destFile = new File(System.getProperty(SCRIPT_FILE_NAME));
		if (destFile == null) {
			usage("You must specify a destination file.");
		}
		//get the mapperName
		mapperName = System.getProperty(SCRIPT_MAPPER_NAME);
		if (mapperName == null) {
			usage("You must specify a mapper name.");
		}
		//get the delete
		String deleteString = System.getProperty(SCRIPT_DELETE_STATEMENT);
		if (deleteString != null) {
			generateDelete = new Boolean(deleteString).booleanValue();
		}
		//get the drop
		String dropString = System.getProperty(SCRIPT_DROP_STATEMENT);
		if (dropString != null) {
			generateDrop = new Boolean(dropString).booleanValue();
		}
		//get the create
		String createString = System.getProperty(SCRIPT_CREATE_STATEMENT);
		if (createString != null) {
			generateCreate = new Boolean(createString).booleanValue();
		}
	}
	
	private int initClass(String className, 
			RdbScriptPMSM msm, 
			ClassLoader classLoader) {
		java.lang.Class currentClass = null;
		try {
			//load the PClassMapping
			currentClass = classLoader.loadClass(className);
			//create a new instance
			Object o = currentClass.newInstance();
			//init the PClassMapping
			PClassMapping pcm = (PClassMapping) o;
			pcm.init(msm);
		} catch (ClassNotFoundException e) {
			log(
					"[ERROR] Impossible to load the persistent class '"
					+ className + "' from the classpath: " + e.getMessage(), Project.MSG_ERR);
			return -1;
		} catch (IllegalAccessException iae) {
			log("[ERROR] Impossible to have access to a new instance of the class '"
					+ className + ": " + iae.getMessage(), Project.MSG_ERR);
			return -1;
		} catch (InstantiationException ie) {
			log("[ERROR] Impossible to get a new instance of the class '"
					+ className + ": " + ie.getMessage(), Project.MSG_ERR);
			return -1;
		} catch (PException pe) {
			log("[ERROR] Impossible to init the PClassMapping class '"
					+ className + ": " + pe.getMessage(), Project.MSG_ERR);
			return -1;
		}
		return 1;
	}
	
	private void usage(String msg) {
		log(msg, Project.MSG_ERR);
		log("Usage: ", Project.MSG_ERR);
		log("\tRdbScriptGeneration (<class name to generate>)*", Project.MSG_ERR);
		log("\t\t-D"+SCRIPT_CLASS_DIR+"=<generated classes directory>", Project.MSG_ERR);
		log("\t\t-D"+SCRIPT_JAR_DIR+"=<added jar directory>", Project.MSG_ERR);
		log("\t\t[-D"+SCRIPT_FILE_NAME+"=<file name>]", Project.MSG_ERR);
		log("\t\t[-D"+SCRIPT_MAPPER_NAME+"=<mapper name>]", Project.MSG_ERR);
		log("\t\t[-D"+SCRIPT_DELETE_STATEMENT+"=<true|false>]", Project.MSG_ERR);
		log("\t\t[-D"+SCRIPT_DROP_STATEMENT+"=<true|false>]", Project.MSG_ERR);
		log("\t\t[-D"+SCRIPT_CREATE_STATEMENT+"=<true|false>]", Project.MSG_ERR);
		System.exit(1);
	}
	
	/**
	 * Remove the file extension
	 * and replace the separator / or \ by . 
	 * for element i of the list
	 */
	public static String formatElement(String element) {
	  	int index = element.lastIndexOf(".");
	   	if (index != -1) {
	   		//	remove the file extension
	   		element = element.substring(0, index);
	   	}
	   	if (element.indexOf("\\") != -1) {
	   		//for windows path
	   		element = replaceString("\\", ".", element);
	   	} else {
	  		//for unix path
	  		element = replaceString("/", ".", element);
	   	}
	   	return element;
	}
	
    public static String replaceString(String old, String neo, String str) {
        if (str == null || str.length() == 0) {
            return str;
        }
        int oldSize = old.length();
        int neoSize = neo.length();
        StringBuffer sb = new StringBuffer(str);
        int begin = 0;
        int idx = sb.indexOf(old, begin);
        while(idx != -1) {
            sb.delete(idx, idx + oldSize);
            sb.insert(idx, neo);
            begin = idx + neoSize;
            if (begin < sb.length()) {
                idx = sb.indexOf(old, begin);
            } else {
                idx = -1;
            }
        }
        return sb.toString();
    }
}
