/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.verifier.lib;

import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.verifier.api.MappingVerifier;
import org.objectweb.jorm.verifier.api.Verifier;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.jorm.util.api.Loggable;

import java.util.HashMap;
import java.util.Collection;

public class JormVerifier implements Verifier, Loggable {
	/**
	 * The meta information manager for managing meta-objects within the
	 * context of a mapper.
	 */
	private Manager metaInfoManager;

	/**
	 * The project name associated with this verifier.
	 */
	private String projectName;

	/**
	 * A simple logger to log.
	 */
	private Logger logger;

	/**
	 * A logger factory to create other loggers if needed.
	 */
	private LoggerFactory loggerFactory;

	/**
	 * the mapping verifier to use to verify the mapping part of the mapping
	 * information.
	 */
	private HashMap mappingVerifiers = new HashMap();

	/**
	 * builds a new JormVerifier object.
	 */
	public JormVerifier() {
	}

	// IMPLEMENTATION OF METHODS FROM THE Verifier INTERFACE

	/**
	 * Assigns the meta information manager created by the parser module to
	 * the current BlobVerifier object.
	 *
	 * @param mim   the meta information manager to use
	 */
	public void setMetaInfoManager(Manager mim) {
		metaInfoManager = mim;
	}

	/**
	 * Assigns a project name to the verifier.
	 * @param name	The associated name.
	 */
	public void setProjectName(String name) {
		projectName = name;
	}

	/**
	 * Sets the mapping verifier which verifies the mapping part of the meta
	 * information.
	 * @param mappername		the name of the mapper
	 * @param mappingVerifier   the Mapping BlobVerifier object
	 */
	public void addMappingVerifier(String mappername, MappingVerifier mappingVerifier) {
		mappingVerifiers.put(mappername, mappingVerifier);
	}

	/**
	 * Verifies the meta information.
	 */
	public void verify(Collection mos) {
		if (logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG,
					   "Start the verification of the meta-information");

		// work

		if (logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG,
					   "The verification of the meta-information is finished");
	}

	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger logger) {
		this.logger = logger;
	}

	public void setLoggerFactory(LoggerFactory loggerfactory) {
		this.loggerFactory = loggerfactory;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
}
