/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.xml2mi.api;

import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.util.io.api.PathExplorer;
import org.objectweb.jorm.api.PException;
import org.w3c.dom.Element;

import java.util.Map;
import java.util.Hashtable;

/**
 * The MappingParser interface provides methods to parse specific mapping
 * information from an XML Element object and build the corresponding Jorm
 * metainformation.
 */
public interface MappingParser {
	/**
	 * Assigns a meta-information manager to a parser in order from it to
	 * create the meta-information related to the XML files it parses.
	 * @param manager	the meta-information Manager to be associated to this
	 * 				parser
	 */
	void setMetaInfoManager(Manager manager);

	/**
	 * Assigns a PathExplorer object for locating files that have to be parsed.
	 * @param explorer	The PathExplorer to be used for file location
	 */
	void setPathExplorer(PathExplorer explorer);

	/**
	 * Assigns the current Class object to a parser in order from it to
	 * retrieve the meta-objects (for example, PrimitiveElement objects)
	 * that are referenced from XML mapping elements.
	 * @param aClass	the Jorm meta-object associated to the parsed class.
	 */
	void setCurrentClass(Class aClass);

	/**
	 * Parses the mapping information related to a particular mapper and builds
	 * the corresponding Jorm meta-information. It can be class-related or
	 * generic class-related information.
	 * Manager, PathExplorer and currentClass must be set before calling this
	 * method.
	 * @param mappingElem   the current XML node.
	 * @param mapping       a Mapping object.
	 */
	void parseMapping(Element mappingElem, Mapping mapping) throws PException;

	/**
	 * @param idvalue2genclassref	an HashTable that associates an id value with a GenClassRef object.
	 */
	void setidvalue2genclassref(Map idvalue2genclassref);

	/**
	 * Assigns an HashTable object to the MappingParser object.
	 * @param motable	an HashTable that associates a pd file to its corresponding MetaObject.
	 */
	void setmotable(Hashtable motable);
}
