/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.xml2mi.lib;

import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.ClassMapping;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.io.File;
import java.util.Hashtable;

/**
 * This class is a common helper between generic and specific parsers.
 *
 * @author S.Chassande-Barrioz
 */
public class ParserHelper implements Loggable {
	/**
	 * This structure contains Class and CompositeName objects.
	 * key = a JORM persistent definition file
	 * value = the associated MetaObject object.
	 */
	protected Hashtable motable = null;

	/**
	 * a simple logger to log
	 */
	protected Logger logger;

	/**
	 * a logger factory to create other loggers if needed
	 */
	protected LoggerFactory loggerFactory;


	/**
	 * Returns a NameDef object given a NameDef name (name) and possibly the full qualified
	 * class name (fqcn) where it is defined. This NameDef object is referenced from an
	 * IdentifierMapping object.
	 * @param linkend	a String object whose structure is defined as follows: /fqcn/name or /fqcn or name
	 * @return a NameDef object.
	 */
	protected NameDef getIdNameDef(Class currentClass, String linkend) {
		NameDef nameDef = null;
		int idx1 = linkend.indexOf("/");
		if (idx1 == 0) { // link-end value: /fqcn/name   or /fqcn
            String fqcn = null;
		    String namedefName = null;
			int idx2 = linkend.indexOf("/", idx1 + 1);
            if (idx2 == -1) { // link-end value: /fqcn
			    fqcn = linkend.substring(1);
			    namedefName = "";
            } else { // link-end value: /fqcn/name
			    fqcn = linkend.substring(1, idx2);
			    namedefName = linkend.substring(idx2 + 1);

            }
			if (logger.isLoggable(BasicLevel.DEBUG)) {
		        logger.log(BasicLevel.DEBUG,
                           " fqcn " + fqcn +
						   " namdefname " + namedefName);
            }
			fqcn = fqcn.replace('.', File.separatorChar) + ".pd";
			Class clazz = (Class) motable.get(fqcn);
			if (clazz != null) {
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG,
							   "Fetching the id name def of the Class " + clazz.getFQName());
				}
				nameDef = clazz.getNameDef(namedefName);
			}

        } else { // link-end value: name
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG,
						   "Fetching the id name def of the current Class " + currentClass.getFQName());
			}
			nameDef = currentClass.getNameDef(linkend);
        }

		if ((nameDef != null) && (logger.isLoggable(BasicLevel.DEBUG))) {
			if (nameDef.isFieldName())
				logger.log(BasicLevel.DEBUG, "fieldName: " + nameDef.getFieldName());
			else if (nameDef.isNameRef())
				logger.log(BasicLevel.DEBUG, "NameRef: " + nameDef.getNameRef());
		}
		return nameDef;
	}

	protected NameDef getIdNameDef(ClassMapping classMapping, String linkend) {
		return getIdNameDef((Class) classMapping.getLinkedMO(), linkend);
	}

	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	/**
	 * Defines a logger object.
	 *
	 * @param logger    the logger object
	 */
	public void setLogger(Logger logger) {
		this.logger = logger;
	}

	/**
	 * Defines the logger factory to obtain new loggers.
	 *
	 * @param loggerfactory     The LoggerFactory object to obtain a logger object
	 */
	public void setLoggerFactory(LoggerFactory loggerfactory) {
		this.loggerFactory = loggerfactory;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
}
