/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */
package org.objectweb.medor.filter.jorm.lib;

import org.objectweb.medor.api.Field;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.IExpressionPrinter;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.filter.api.FieldOperand;

/**
 * Is printer of expression containing JORM stuffs.
 * 
 * @author S.Chassande-Barrioz
 */
public class ExpressionPrinter implements IExpressionPrinter {

    static {
        org.objectweb.medor.expression.lib.ExpressionPrinter
            .registerPrinter(new ExpressionPrinter());
    }
    
    public String expToString(Expression e) {
        if (e instanceof CompositePName) {
            FieldOperand[] fields = ((CompositePName) e).getFields();
            String result = "CompositePName(";
            Field f;
            for(int i=0; i<fields.length; i++) {
                f = fields[i].getField();
                if (f==null)
                    result += "no field ,";
                else
                    result += f.getName() + ", ";
            }
            result += ((CompositePName) e).getExpression(0);
            result += ")";
            return result;
        } else if (e instanceof SinglePName) {
            String res = "SinglePName(";
            Operand op = (Operand) ((SinglePName) e) .getExpression(1);
            Field f = ((SinglePName) e).getField().getField();
            res += (f == null ? null : f.getName()) + ", ";
            if (op instanceof ParameterOperand) {
                res += ((ParameterOperand) op).getName();
            } else {
                res += op.getObject();
            }
            res += ")";
            return res;
        } else 
            return null;
    }

}
