/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2005 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, S. Chassande-Barrioz, A. Lefebvre
 */
package org.objectweb.medor.optim.jorm;

import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.jorm.lib.CompositePName;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.optim.lib.DropUselessNodeRule;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Map;

/**
 * Is an extension of DropUselessNodeRule rule managing QueryTree with JORM
 * stuffs.
 *
 * @author S.Chassande-Barrioz
 */
public class JormDropUselessNodeRule extends DropUselessNodeRule {

    public JormDropUselessNodeRule() {
        super();
    }
    protected boolean replaceInFilter(Expression e, Map map) throws MedorException {
        if (e instanceof CompositePName) {
            FieldOperand[] fs = ((CompositePName) e).getFields();
            boolean replaced = false;
            for (int i = 0; i < fs.length; i++) {
                QueryTreeField qtf = (QueryTreeField) fs[i].getField();
                Field neo = replaceField(qtf, map);
                if (neo != qtf) {
                    fs[i].setField(neo);
                    replaced = true;
                    if (debug) log.log(BasicLevel.DEBUG, "Replace in CompositePName " + qtf.getName() + " by " + neo.getName());
                }
            }
            return replaced;
        } else if (e instanceof SinglePName) {
            FieldOperand fo = ((SinglePName) e).getField();
            QueryTreeField qtf = (QueryTreeField) fo.getField();
            Field neo = replaceField(qtf, map);
            if (neo != qtf) {
                fo.setField(neo);
                if (debug) log.log(BasicLevel.DEBUG, "Replace in SinglePName " + qtf.getName() + " by " + neo.getName());
                return true;
            }
        }
        return super.replaceInFilter(e, map);
    }
}
