/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2005 France Telecom
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */
package org.objectweb.medor.query.jorm.lib;

import org.objectweb.jorm.api.PMapper;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.lib.BasicTupleStructure;
import org.objectweb.medor.query.api.OrderField;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.jorm.api.JormExtent;
import org.objectweb.medor.query.jorm.api.JormField;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * This class is the common implementation of the jorm leaves (ClassExtent,
 * GenClassExtent).
 *
 * @author S.Chassande-Barrioz
 */
public abstract class BasicJormExtent
	extends BasicTupleStructure
	implements JormExtent, QueryLeaf, TupleStructure {
	/**
	 * Node name
	 */
	protected String name;

	/**
	 * Name of the field which is the PName of the extent
	 */
	protected String pnFieldName;

	protected PNameField identifier = null;

	/**
	 * The datastore of the extent
	 */
	protected DataStore myStore;

	/**
	 * The mapper of the extent.
	 */
	protected PMapper mapper;

	protected String projectName = null;

	/**
	 * Are objects of subclasses included.
	 */
	protected boolean includeSubclasses = true;
	
	protected Map annotations = new HashMap();

	public BasicJormExtent() {
	}

	public BasicJormExtent(String _name, String pnFieldName) {
		this.name = (_name == null ? "" : _name);
		this.pnFieldName = pnFieldName;
	}

	public Object clone(Object clone,
						Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		BasicJormExtent bje = (BasicJormExtent) clone;
		bje.name = name;
		bje.pnFieldName = pnFieldName;
		bje.identifier = (PNameField) getClone(identifier, obj2clone);
		bje.myStore = myStore;
		bje.mapper = mapper;
		bje.projectName = projectName;
		bje.includeSubclasses = includeSubclasses;
		return clone;
	}

	public Iterator iterateFields() {
		return fields.iterator();
	}

	public String getFieldName(String nodeName, String fieldName) {
		if (nodeName == null || nodeName.length() == 0)
			return fieldName;
		else
			return nodeName + "." + fieldName;
	}

	// IMPLEMENTATION OF THE JormExtent INTERFACE //
	//--------------------------------------------//

	public PMapper getPMapper() {
		return mapper;
	}

	public void setPMapper(PMapper m, String projectName) {
		mapper = m;
		this.projectName = projectName;
	}

	public String getProjectName() {
		return projectName;
	}

	public void setProjectName(String projectName) {
		this.projectName = projectName;
	}

	public void setDataStore(DataStore ds) {
		myStore = ds;
	}

	public String getPNameFieldName() {
		return getFieldName(name, pnFieldName);
	}

	public abstract String getJormName();

	public JormField getIdentifierField() {
		return identifier;
	}

	public boolean withSubClasses() {
		return includeSubclasses;
	}

	public void setWithSubClasses(boolean incSubClasses) {
		includeSubclasses =  incSubClasses;
	}

	// IMPLEMENTATION OF THE QueryLeaf INTERFACE //
	//-------------------------------------------//

	public String getName() {
		return this.name;
	}

	/**
	 * Methods inherited from QueryTree
	 */
	public TupleStructure getTupleStructure() {
		return this;
	}

	/**
	 * Methods inherited from QueryLeaf
	 */
	public DataStore getDataStore() {
		return myStore;
	}

	public void setDistinct(boolean d) throws MedorException {
		throw new MedorException("Cannot set distinct on a JORM Leaf");
	}

	public boolean getDistinct() {
		return false;
	}

	public void setOrderBy(OrderField[] orderfields) throws MedorException {
		throw new MedorException("Order by not supported on JormExtent");
	}

	public OrderField[] getOrderBy() {
		return null;
	}

	public Map getAnnotations() {
	    return annotations;
	}
}
